<#ibiztemplate>
FILENAME=${item.getPKGCodeName()}Conf.java
PUBOBJ=SysMSAppSpringBootMain
%PUBPKG%=${pub.getPKGCodeName()?replace('.','/')}
%PKG%=${item.getPKGCodeName()?lower_case}
</#ibiztemplate>
package ${pub.getPKGCodeName()}.ms.springboot.app.${item.getPKGCodeName()?lower_case};

import java.io.IOException;
import java.util.Properties;

import javax.sql.DataSource;

import org.hibernate.SessionFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
<#if ( sfstyle.getStyleParam("springboot.version","2.0.1.RELEASE") == '2.0.1.RELEASE')>
import org.springframework.boot.jdbc.DataSourceBuilder;
<#elseif ( sfstyle.getStyleParam("springboot.version","2.0.1.RELEASE") == '1.5.13.RELEASE')>
import org.springframework.boot.autoconfigure.jdbc.DataSourceBuilder;
</#if>
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.io.Resource;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;
import org.springframework.core.io.support.ResourcePatternResolver;
import org.springframework.orm.hibernate5.HibernateTransactionManager;
import org.springframework.orm.hibernate5.LocalSessionFactoryBean;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.EnableTransactionManagement;

import lombok.Data;
import net.ibizsys.paas.db.impl.DBDialectImpl;
import net.ibizsys.paas.util.ObjectHelper;
import net.ibizsys.paas.util.StringHelper;
import net.ibizsys.paas.util.SystemRTHelper;



@Configuration
@EnableTransactionManagement
@EnableConfigurationProperties({
        ${item.getPKGCodeName()}Conf.C3p0Properties.class,
        ${item.getPKGCodeName()}Conf.HibernateProperties.class
})
public class ${item.getPKGCodeName()}Conf {


    @Autowired
    private C3p0Properties c3p0Properties;

    @Autowired
    private HibernateProperties hibernateProp;

    @Bean
    @Qualifier("dataSource")
    @ConfigurationProperties(prefix = "datasource.c3p0")
    public DataSource dataSource() {
        return DataSourceBuilder.create()
                .type(com.mchange.v2.c3p0.ComboPooledDataSource.class)
                .driverClassName(c3p0Properties.getDriverClass())
                .url(c3p0Properties.getJdbcUrl())
                .username(c3p0Properties.getUser())
                .password(c3p0Properties.getPassword())
                .build();
    }

    @Bean(name="dbDialect")
    public DBDialectImpl dbDialect(){
        try {
			return (DBDialectImpl) ObjectHelper.create(c3p0Properties.getIbzDialect());
		} catch (Exception e) {
			e.printStackTrace();
		}
        return null ;
    }

    @Bean
    public LocalSessionFactoryBean sessionFactory(DataSource dataSource) {
        LocalSessionFactoryBean sessionFactory = new LocalSessionFactoryBean();
        sessionFactory.setDataSource(dataSource);
        Properties hibernateProperties = new Properties();
        hibernateProperties.put("hibernate.dialect", hibernateProp.getDialect());
        hibernateProperties.put("hibernate.show_sql", hibernateProp.getShowSql());
        hibernateProperties.put("hibernate.hbm2ddl.auto", hibernateProp.getDdlAuto());
        hibernateProperties.put("hibernate.current_session_context_class",hibernateProp.getCurrentSessionContextClass());
        sessionFactory.setHibernateProperties(hibernateProperties);
        return sessionFactory;
    }

    @Bean
    public HibernateTransactionManager transactionManager(SessionFactory sessionFactory) {
        HibernateTransactionManager txManager = new HibernateTransactionManager();
        txManager.setSessionFactory(sessionFactory);
        return txManager;
    }


    @Value("${r'${ibiz.installDBModels:}'}")
	String strInstallDBModels;
    
    @Component
    public class InstallDBCommandLineRunner implements CommandLineRunner{

        @Override
        public void run(String... var1) throws Exception{
        	if(!StringHelper.isNullOrEmpty(strInstallDBModels)){
				SystemRTHelper.installDBModel(strInstallDBModels, false);
			}
<#if sys.isEnableDynaSys()??>
			//安装动态系统
        	SystemRTHelper.installDynaSys();
</#if>
       }
    }

    @ConfigurationProperties(prefix = "hibernate")
    @Data
    static class HibernateProperties{
        private String dialect;
        private String ddlAuto;
        private String showSql;
        private String currentSessionContextClass;
    }

    @ConfigurationProperties(prefix = "datasource.c3p0")
    @Data
    static class C3p0Properties {
        private String driverClass;
        private String jdbcUrl;
        private String user;
        private String password;
        private int maxPoolSize;
        private int minPoolSize;
        private int initialPoolSize;
        private int maxIdleTime;
        private int acquireIncrement;
        private int maxStatements;
        private int idleConnectionTestPeriod;
        private String ibzDialect;
    }

<#if sys.hasPSWFEngineType("ACTIVITI")>
	@Bean
	public org.activiti.spring.SpringProcessEngineConfiguration processEngineConfiguration(HibernateTransactionManager transactionManager, DataSource dataSource) {
		org.activiti.spring.SpringProcessEngineConfiguration processEngineConfiguration = new org.activiti.spring.SpringProcessEngineConfiguration();
		processEngineConfiguration.setDataSource(dataSource);
		processEngineConfiguration.setTransactionManager(transactionManager);
		processEngineConfiguration.setDatabaseSchemaUpdate("true");
		ResourcePatternResolver resolver = new PathMatchingResourcePatternResolver();
		Resource[] resources = null;
		try {
			resources = resolver.getResources("classpath*:/activiti/*.bpmn");
		} catch (IOException e) {
			e.printStackTrace();
		}
		processEngineConfiguration.setDeploymentResources(resources);
		processEngineConfiguration.setDeploymentMode("single-resource");
		return processEngineConfiguration;
	}

    @Bean
    public org.activiti.spring.ProcessEngineFactoryBean processEngine(org.activiti.spring.SpringProcessEngineConfiguration processEngineConfiguration) {
    	org.activiti.spring.ProcessEngineFactoryBean processEngine = new org.activiti.spring.ProcessEngineFactoryBean();
    	processEngine.setProcessEngineConfiguration(processEngineConfiguration);
        return processEngine;
    }
    
    @Bean
    public org.activiti.engine.RepositoryService repositoryService(org.activiti.engine.ProcessEngine processEngine){
    	return processEngine.getRepositoryService();
    }
    @Bean
    public org.activiti.engine.RuntimeService runtimeService(org.activiti.engine.ProcessEngine processEngine){
    	return processEngine.getRuntimeService();
    }
    @Bean
    public org.activiti.engine.TaskService taskService(org.activiti.engine.ProcessEngine processEngine){
    	return processEngine.getTaskService();
    }
    @Bean
    public org.activiti.engine.FormService formService(org.activiti.engine.ProcessEngine processEngine){
    	return processEngine.getFormService();
    }
    @Bean
    public org.activiti.engine.HistoryService historyService(org.activiti.engine.ProcessEngine processEngine){
    	return processEngine.getHistoryService();
    }
    @Bean
    public org.activiti.engine.IdentityService identityService(org.activiti.engine.ProcessEngine processEngine){
    	return processEngine.getIdentityService();
    }
    @Bean
    public org.activiti.engine.ManagementService managementService(org.activiti.engine.ProcessEngine processEngine){
    	return processEngine.getManagementService();
    }
</#if>    
}