<#ibiztemplate>
FILENAME=${item.codeName}APIClientModelBase.java
PUBOBJ=SubSysAPICode
MODELS=PSSUBSYSSERVICEAPI
%PUBPKG%=${pub.getPKGCodeName()?replace('.','/')}
</#ibiztemplate>
package ${pub.getPKGCodeName()}.srv.api;

import java.io.IOException;
import java.util.ArrayList;

import org.apache.http.client.ClientProtocolException;
import org.springframework.cloud.openfeign.ribbon.LoadBalancerFeignClient;
import org.springframework.core.env.ConfigurableEnvironment;

import feign.Client;
import feign.Contract;
import feign.Feign;
import feign.RequestInterceptor;
import feign.RequestTemplate;
import feign.codec.Decoder;
import feign.codec.Encoder;
import net.ibizsys.paas.api.FetchResult;
import net.ibizsys.paas.api.IRestServiceAPIAction;
import net.ibizsys.paas.api.IServiceCallContext;
import net.ibizsys.paas.api.RestCallException;
import net.ibizsys.paas.api.RestCallResult;
import net.ibizsys.paas.api.RestServiceAPIActionModel;
import net.ibizsys.paas.core.DEDataSetFetchContext;
import net.ibizsys.paas.core.Errors;
import net.ibizsys.paas.core.IDEDataSetFetchContext;
import net.ibizsys.paas.data.DataObject;
import net.ibizsys.paas.db.ISelectCond;
import net.ibizsys.paas.db.impl.SimpleDataRowImpl;
import net.ibizsys.paas.entity.IEntity;
import net.ibizsys.paas.entity.SimpleEntity;
import net.ibizsys.paas.exception.ErrorException;
import net.ibizsys.paas.util.JSONObjectHelper;
import net.ibizsys.paas.util.StringHelper;
import net.sf.json.JSONObject;
import net.ibizsys.paas.sysmodel.ISystemModel;
import net.ibizsys.paas.sysmodel.SysModelGlobal;
import ${pub.getPKGCodeName()}.srv.${sys.codeName}SysModel;
import net.ibizsys.paas.api.RestServiceAPIActionModel;

/**
 * 子系统服务API客户端[${item.name}]基类
 *
 * !! 不要对此代码进行修改
 */
<#if item.getHandler()?? && item.getHandler()?length gt 0>
public abstract class ${item.codeName}APIClientModelBase extends ${item.getHandler()} {
<#else>
<#if item.getAPIType() == 'RESTFUL'>
public abstract class ${item.codeName}APIClientModelBase extends ${pub.getBaseClassPKGCodeName()}.paas.api.RestServiceAPIClientModelBase {
</#if>
</#if>
	public ${item.codeName}APIClientModelBase() throws Exception {
	
	}

	public ${item.codeName}APIClientModelBase(Decoder decoder, Encoder encoder, Client client, Contract contract,ConfigurableEnvironment env) throws Exception {
		super();
		this.setId("${item.id}");
		this.setName("${item.name}");
		this.setUniqueTag("${item.codeName}");
		<#if item.getDENames()?? && item.getPSSubSysServiceAPIMethods()??>   
		prepareAPIActions();
		String strServiceName = env.getProperty("ibiz.client.${item.codeName?lower_case}.service","");
        // nameBuilder直接使用client，它会使用负载均衡
    	if(!StringHelper.isNullOrEmpty(strServiceName)) {
        	Feign.Builder nameBuilder = Feign.builder()
                    .client(client)
                    .encoder(encoder)
                    .decoder(decoder)
                    .contract(contract)
<#if sys.getSaaSMode()==1>
				     .requestInterceptor(new RequestInterceptor() {
        				@Override
        				public void apply(RequestTemplate requestTemplate) {
        			        	requestTemplate.header(net.ibizsys.paas.api.RestServiceAPIClientModelBase.MODULEID, get${sys.codeName}SysModel().getModuleId()) ;
        				}
        			})
</#if>
                    ;
        	${srfmethodname('${item.codeName}')}APIClientRemoteService = nameBuilder.target(${item.codeName}APIClientRemoteService.class,"http://" + strServiceName) ;
    	}

        String strUrl = env.getProperty("ibiz.client.${item.codeName?lower_case}.url","");
    	if(!StringHelper.isNullOrEmpty(strUrl)) { 
    		   if (client instanceof LoadBalancerFeignClient) { // 无需均衡负载
    	            client = ((LoadBalancerFeignClient)client).getDelegate();
    	        }
    	        Feign.Builder urlBuilder = Feign.builder()
    	            .client(client)
    	            .encoder(encoder)
    	            .decoder(decoder)
    	            .contract(contract)
<#if sys.getSaaSMode()==1>
				     .requestInterceptor(new RequestInterceptor() {
        				@Override
        				public void apply(RequestTemplate requestTemplate) {
        			        	requestTemplate.header(net.ibizsys.paas.api.RestServiceAPIClientModelBase.MODULEID, get${sys.codeName}SysModel().getModuleId()) ;
        				}
        			})
</#if>
    				;
    	        ${srfmethodname('${item.codeName}')}APIClientRemoteService = urlBuilder.target(${item.codeName}APIClientRemoteService.class, strUrl) ;
    	}
		</#if>  
	}
<#if item.getHandler()?? && item.getHandler()?length gt 0>
<#else>

    private ${item.codeName}APIClientRemoteService ${srfmethodname('${item.codeName}')}APIClientRemoteService ;
    
    @Override
	public void execute(IServiceCallContext iServiceCallContext, String strActionTag, IEntity iEntity) throws Exception {

		IRestServiceAPIAction iRestServiceAPIAction = (IRestServiceAPIAction) this.getServiceAPIAction(strActionTag);
		String strUrl = StringHelper.format("%1$s%2$s", this.getServicePath(iRestServiceAPIAction,iEntity), iRestServiceAPIAction.getActionPath());

		
		String retStr = null;
		try {
			JSONObject json = new JSONObject() ;
			iEntity.fillJSONObject(json, true, true) ;
	
<#if item.getPSSubSysServiceAPIMethods()??>   
<#assign outfalg=false>
<#list item.getPSSubSysServiceAPIMethods() as action>
<#assign restapi=action.getPSRESTfulAPI()>
<#if action.name == "GetDraft">
			<#if outfalg == true>else </#if>if(strActionTag.equals("${action.getUniqueTag()}")) {
				retStr = ${srfmethodname('${item.codeName}')}APIClientRemoteService.${srfmethodname('${action.name}')}${action.getPSDECodeName()}() ;
			}
<#assign outfalg=true>
<#elseif restapi.getRequestMethod()=="GET" || restapi.getRequestMethod()=="DELETE">
			<#if outfalg == true>else </#if>if(strActionTag.equals("${action.getUniqueTag()}")) {
				String strKeyField = iRestServiceAPIAction.getKeyField();
				if(StringHelper.isNullOrEmpty(strKeyField))
				{
					strKeyField = IEntity.KEY;
				}
				Object objKey = iEntity.get(strKeyField);
				retStr = ${srfmethodname('${item.codeName}')}APIClientRemoteService.${srfmethodname('${action.name}')}${action.getPSDECodeName()}(objKey.toString()) ;
			}
<#assign outfalg=true>
<#elseif restapi.getActionType() != "FETCH" && restapi.getActionType() != "FETCHTEMP" && restapi.getActionType() != "SELECT" && restapi.getActionType() != "SELECTTEMP">
			<#if outfalg == true>else </#if>if(strActionTag.equals("${action.getUniqueTag()}")) {
				retStr = ${srfmethodname('${item.codeName}')}APIClientRemoteService.${srfmethodname('${action.name}')}${action.getPSDECodeName()}(json.toString()) ;
			}
<#assign outfalg=true>
</#if>
</#list>
</#if>     

//			retStr = httpclient.execute(httpUriRequest, responseHandler);
		} catch (ClientProtocolException ex) {
			throw ex;
		} catch (IOException ex) {
			throw ex;
		}

		if (StringHelper.isNullOrEmpty(retStr)) {
			throw new RestCallException(Errors.INTERNALERROR, "没有返回任何内容");
		}

		RestCallResult restCallResult = new RestCallResult();
		try {
			JSONObject jo = JSONObjectHelper.fromString(retStr);
			restCallResult.fromJSONObject(jo);
			if(iServiceCallContext!=null){
				iServiceCallContext.setResultJO(jo);
				iServiceCallContext.setResultRaw(retStr);
			}

		} catch (Exception ex) {
			throw new RestCallException(Errors.INTERNALERROR, "解析返回内容发生异常");
		}

		if (restCallResult.getRetCode() != Errors.OK) {
			throw new ErrorException(restCallResult.getRetCode(), restCallResult.getErrorInfo(false));
		}

		if ((StringHelper.compare(iRestServiceAPIAction.getRequestMethod(), IRestServiceAPIAction.REQUESTMETHOD_GET, false) == 0) || (StringHelper.compare(iRestServiceAPIAction.getRequestMethod(), IRestServiceAPIAction.REQUESTMETHOD_POST, false) == 0) || (StringHelper.compare(iRestServiceAPIAction.getRequestMethod(), IRestServiceAPIAction.REQUESTMETHOD_PUT, false) == 0)) {
			JSONObject item = restCallResult.getItem(false);
			if (item != null) {
				DataObject.fromJSONObject(iEntity, item, false, false);
			} else {
				throw new RestCallException(Errors.INTERNALERROR, "返回内容中没有包含数据");
			}
		}
	}	
	
	
	@Override
    public ArrayList<IEntity> select(IServiceCallContext iServiceCallContext, String strActionTag,
    		ISelectCond iSelectCond) throws Exception {
    	IRestServiceAPIAction iRestServiceAPIAction = (IRestServiceAPIAction) this.getServiceAPIAction(strActionTag);
		String strUrl = StringHelper.format("%1$s%2$s", this.getServicePath(iRestServiceAPIAction,iSelectCond), iRestServiceAPIAction.getActionPath());
		
		String retStr = null;
		
		try {
			JSONObject json = this.getSelectCondJSONObject(iSelectCond, iServiceCallContext, iRestServiceAPIAction);
			<#if item.getPSSubSysServiceAPIMethods()??>  
			<#assign outfalg=false> 
			<#list item.getPSSubSysServiceAPIMethods() as action>
			<#assign restapi=action.getPSRESTfulAPI()>
			<#if restapi.getActionType()=="SELECT">
						<#if outfalg == true>else </#if>if(strActionTag.equals("${action.getUniqueTag()}")) {
							retStr = ${srfmethodname('${item.codeName}')}APIClientRemoteService.${action.name?lower_case}${action.getPSDECodeName()}(json.toString()) ;
						}
			<#assign outfalg=true>
			</#if>
			</#list>
			</#if>  
		} catch (ClientProtocolException ex) {
			throw ex;
		} catch (IOException ex) {
			throw ex;
		}

		if (StringHelper.isNullOrEmpty(retStr)) {
			throw new RestCallException(Errors.INTERNALERROR, "没有返回任何内容");
		}

		RestCallResult restCallResult = this.createRestCallResult();
		try {
			JSONObject jo = JSONObjectHelper.fromString(retStr);
			restCallResult.fromJSONObject(jo);
			if(iServiceCallContext!=null){
				iServiceCallContext.setResultJO(jo);
				iServiceCallContext.setResultRaw(retStr);
			}

		} catch (Exception ex) {
			throw new RestCallException(Errors.INTERNALERROR, "解析返回内容发生异常");
		}

		if (restCallResult.getRetCode() != Errors.OK) {
			throw new ErrorException(restCallResult.getRetCode(), restCallResult.getErrorInfo(false));
		}

		if ((StringHelper.compare(iRestServiceAPIAction.getRequestMethod(), IRestServiceAPIAction.REQUESTMETHOD_GET, false) == 0) || (StringHelper.compare(iRestServiceAPIAction.getRequestMethod(), IRestServiceAPIAction.REQUESTMETHOD_POST, false) == 0) || (StringHelper.compare(iRestServiceAPIAction.getRequestMethod(), IRestServiceAPIAction.REQUESTMETHOD_PUT, false) == 0)) {
			ArrayList<JSONObject> items = restCallResult.getItems(false);
			if (items != null) {
				ArrayList<IEntity> list = new ArrayList<IEntity>();
				for (JSONObject jo : items) {
					SimpleEntity simpleEntity = new SimpleEntity();
					this.fromJSONObject(simpleEntity, jo, iServiceCallContext, iRestServiceAPIAction);
					list.add(simpleEntity);
				}
				return list;
			}
			else{
				return new ArrayList<IEntity>();
			}
		}

		throw new RestCallException(Errors.INTERNALERROR, "返回内容中没有包含数据");
    }
   
    @Override
	public FetchResult fetch(IServiceCallContext iServiceCallContext, String strActionTag, IDEDataSetFetchContext iDEDataSetFetchContext) throws Exception {
		IRestServiceAPIAction iRestServiceAPIAction = (IRestServiceAPIAction) this.getServiceAPIAction(strActionTag);
		String strUrl = StringHelper.format("%1$s%2$s", this.getServicePath(iRestServiceAPIAction,iDEDataSetFetchContext), iRestServiceAPIAction.getActionPath());
		
		String retStr = null;
		try {
			JSONObject json = DEDataSetFetchContext.toJSONObject(iDEDataSetFetchContext, null);
<#if item.getPSSubSysServiceAPIMethods()??>  
<#assign outfalg=false> 
<#list item.getPSSubSysServiceAPIMethods() as action>
<#assign restapi=action.getPSRESTfulAPI()>
<#if restapi.getActionType()=="FETCH">
			<#if outfalg == true>else </#if>if(strActionTag.equals("${action.getUniqueTag()}")) {
				retStr = ${srfmethodname('${item.codeName}')}APIClientRemoteService.fetch${action.name}${action.getPSDECodeName()}(json.toString()) ;
			}
<#assign outfalg=true>
<#elseif restapi.getActionType()=="FETCHTEMP">
			<#if outfalg == true>else </#if>if(strActionTag.equals("${action.getUniqueTag()}")) {
				retStr = ${srfmethodname('${item.codeName}')}APIClientRemoteService.fetchTemp${action.name}${action.getPSDECodeName()}(json.toString()) ;
			}
<#assign outfalg=true>
</#if>
</#list>
</#if>     			
		}  catch (Exception ex) {
			throw ex;
		}

		if (StringHelper.isNullOrEmpty(retStr)) {
			throw new RestCallException(Errors.INTERNALERROR, "没有返回任何内容");
		}

		RestCallResult restCallResult = new RestCallResult();
		try {
			JSONObject jo = JSONObjectHelper.fromString(retStr);
			restCallResult.fromJSONObject(jo);
			if(iServiceCallContext!=null){
				iServiceCallContext.setResultJO(jo);
				iServiceCallContext.setResultRaw(retStr);
			}

		} catch (Exception ex) {
			throw new RestCallException(Errors.INTERNALERROR, "解析返回内容发生异常");
		}

		if (restCallResult.getRetCode() != Errors.OK) {
			throw new ErrorException(restCallResult.getRetCode(), restCallResult.getErrorInfo(false));
		}

		if ((StringHelper.compare(iRestServiceAPIAction.getRequestMethod(), IRestServiceAPIAction.REQUESTMETHOD_GET, false) == 0) || (StringHelper.compare(iRestServiceAPIAction.getRequestMethod(), IRestServiceAPIAction.REQUESTMETHOD_POST, false) == 0) || (StringHelper.compare(iRestServiceAPIAction.getRequestMethod(), IRestServiceAPIAction.REQUESTMETHOD_PUT, false) == 0)) {
			FetchResult fetchResult = new FetchResult();
			fetchResult.setTotalRow(restCallResult.getTotalRow());
			ArrayList<JSONObject> items = restCallResult.getItems(false);
			if (items != null) {
				for (JSONObject jo : items) {
					SimpleDataRowImpl simpleDataRowImpl = new SimpleDataRowImpl();
					DataObject.fromJSONObject(simpleDataRowImpl, jo ,false, false);
					fetchResult.getDataRows().add(simpleDataRowImpl);
				}
			}
			return fetchResult;
		}

		throw new RestCallException(Errors.INTERNALERROR, "返回内容中没有包含数据");

	}
    
<#if item.getDENames()?? && item.getPSSubSysServiceAPIMethods()??>     
	protected void prepareAPIActions()throws Exception{
    	 <#list item.getDENames() as dename>
    	 <#if dename?? && dename?length gt 0 >
		prepareAPIActions_${dename}();
    	 <#else>
		prepareAPIActionsUnknown();
    	 </#if>
    	 </#list>
	}
	<#list item.getDENames() as dename>
	<#if dename?? && dename?length gt 0 >
	<#assign unknownflag=0>
	protected void prepareAPIActions_${dename}()throws Exception{
	<#else>
	<#assign unknownflag=1>
	protected void prepareAPIActionsUnknown()throws Exception{
   	 </#if>	
     	<#list item.getPSSubSysServiceAPIMethods() as action>
     	<#assign outputflag=0>
   		<#if action.getDEName()?? && action.getDEName()?length gt 0>
   		<#if unknownflag==0 && action.getDEName() == dename>
   		<#assign outputflag=1>
   		</#if>
   		<#elseif unknownflag==1>
   		<#assign outputflag=1>
   		</#if>
   		<#if outputflag==1>
		RestServiceAPIActionModel action${action_index?c} = new RestServiceAPIActionModel();
		action${action_index?c}.setId("${action.id}");
<#if action.name ?? && action.name?length gt 0>              
		action${action_index?c}.setName("${action.name}");
</#if> 		
		action${action_index?c}.setUniqueTag("${action.getUniqueTag()}");
		action${action_index?c}.setActionType(RestServiceAPIActionModel.ACTIONTYPE_${action.getActionType()});
<#if action.getPSRESTfulAPI().getRequestMethod()??>
		action${action_index?c}.setRequestMethod(RestServiceAPIActionModel.REQUESTMETHOD_${action.getPSRESTfulAPI().getRequestMethod()});
</#if>
<#if action.getPSRESTfulAPI().getRequestPath()??>
		action${action_index?c}.setActionPath("${action.getPSRESTfulAPI().getRequestPath()}");
</#if>
<#if action.getKeyField()?? && action.getKeyField()?length gt 0>
		action${action_index?c}.setKeyField("${action.getKeyField()}");
</#if>
		this.registerServiceAPIAction(action${action_index?c});
		</#if>
</#list>
	}
	</#list>
</#if>     
</#if>

    private ${sys.codeName}SysModel ${srfparamname('${sys.codeName}')}SysModel;
    
    public  ${sys.codeName}SysModel get${sys.codeName}SysModel() {
           if(this.${srfparamname('${sys.codeName}')}SysModel==null)
           {
               try
                {
                      this.${srfparamname('${sys.codeName}')}SysModel = (${sys.codeName}SysModel)SysModelGlobal.getSystem("${pub.getPKGCodeName()}.srv.${sys.codeName}SysModel");
                }
                catch(Exception ex)
                {
                }
           }       
           return this.${srfparamname('${sys.codeName}')}SysModel;
      }


       public  ISystemModel getSystemModel() {
        return this.get${sys.codeName}SysModel();
    }
}